@@ -32,11 +32,12 @@ typedef struct {                         // type of structure for a record
 
 
 /* global variables */
-TCMPOOL *g_mpool;                        // global memory pool
-TCTMPL *g_tmpl;                          // template serializer
-void *g_scrextproc;                      // processor of the script extension
-TCMAP *g_users;                          // user list
-long g_eventcount;                       // event counter
+time_t g_starttime = 0;                  // start time of the process
+TCMPOOL *g_mpool = NULL;                 // global memory pool
+TCTMPL *g_tmpl = NULL;                   // template serializer
+TCMAP *g_users = NULL;                   // user list
+void *g_scrextproc = NULL;               // processor of the script extension
+unsigned long g_eventcount = 0;          // event counter
 const char *g_scriptname;                // script name
 const char *g_scriptprefix;              // script prefix
 const char *g_scriptpath;                // script path
@@ -85,16 +86,21 @@ static bool doupdatecmd(TCMPOOL *mpool, const char *mode, const char *baseurl, c
 /* main routine */
 int main(int argc, char **argv){
 #if defined(MYFCGI)
-  g_mpool = tcmpoolglobal();
+  g_starttime = time(NULL);
+  g_mpool = tcmpoolnew();
   int rv = 0;
   while(FCGI_Accept() >= 0){
     g_eventcount++;
     if(realmain(argc, argv) != 0) rv = 1;
   }
+  tcmpooldel(g_mpool);
   return rv;
 #else
-  g_mpool = tcmpoolglobal();
-  return realmain(argc, argv);
+  g_starttime = time(NULL);
+  g_mpool = tcmpoolnew();
+  int rv = realmain(argc, argv);
+  tcmpooldel(g_mpool);
+  return rv;
 #endif
 }
 
@@ -129,6 +135,13 @@ static int realmain(int argc, char **argv){
     if(*buf != '\0') g_scriptname = buf;
   }
   if(g_tmpl){
+    if(g_users){
+      int64_t mtime;
+      if(tcstatfile(g_password, NULL, NULL, &mtime) && mtime >= g_starttime){
+        tcmapclear(g_users);
+        readpasswd();
+      }
+    }
     dosession(mpool);
   } else {
     g_tmpl = tcmpoolpush(g_mpool, tctmplnew(), (void (*)(void *))tctmpldel);
@@ -136,7 +149,6 @@ static int realmain(int argc, char **argv){
       g_database = tctmplconf(g_tmpl, "database");
       if(!g_database) g_database = "promenade.tct";
       g_password = tctmplconf(g_tmpl, "password");
-      g_users = NULL;
       if(g_password){
         g_users = tcmpoolpushmap(g_mpool, tcmapnew2(TINYBNUM));
         readpasswd();
@@ -157,8 +169,8 @@ static int realmain(int argc, char **argv){
         }
       }
       g_scrext = tctmplconf(g_tmpl, "scrext");
-      g_scrextproc = tcmpoolpush(g_mpool, g_scrext && *g_scrext != '\0' ? scrextnew() : NULL,
-                                 scrextdel);
+      if(g_scrext && *g_scrext != '\0')
+        g_scrextproc = tcmpoolpush(g_mpool, scrextnew(), scrextdel);
       const char *rp = tctmplconf(g_tmpl, "recvmax");
       g_recvmax = rp ? tcatoix(rp) : INT_MAX;
       g_title = tctmplconf(g_tmpl, "title");
@@ -1317,6 +1329,9 @@ static void dosession(TCMPOOL *mpool){
   }
   tcmapput2(vars, "mimetype", mtype);
   const char *tmplstr = tcmpoolpushptr(mpool, tctmpldump(g_tmpl, vars));
+  while(*tmplstr > '\0' && *tmplstr <= ' '){
+    tmplstr++;
+  }
   if(g_scrextproc && scrextcheckfunc(g_scrextproc, "_procpage")){
     char *obuf = tcmpoolpushptr(mpool, scrextcallfunc(g_scrextproc, "_procpage", tmplstr));
     if(obuf) tmplstr = obuf;
