@@ -28,6 +28,7 @@ static void printdberr(TCTDB *tdb);
 static int runcreate(int argc, char **argv);
 static int runimport(int argc, char **argv);
 static int runexport(int argc, char **argv);
+static int runupdate(int argc, char **argv);
 static int runremove(int argc, char **argv);
 static int runconvert(int argc, char **argv);
 static int runpasswd(int argc, char **argv);
@@ -35,6 +36,7 @@ static int runversion(int argc, char **argv);
 static int proccreate(const char *dbpath, int scale, bool fts);
 static int procimport(const char *dbpath, TCLIST *files, TCLIST *sufs);
 static int procexport(const char *dbpath, int64_t id, const char *dirpath);
+static int procupdate(const char *dbpath, int64_t id, const char *wiki);
 static int procremove(const char *dbpath, int64_t id);
 static int procconvert(const char *ibuf, int isiz, int fmt,
                        const char *buri, const char *duri, bool page);
@@ -53,6 +55,8 @@ int main(int argc, char **argv){
     rv = runimport(argc, argv);
   } else if(!strcmp(argv[1], "export")){
     rv = runexport(argc, argv);
+  } else if(!strcmp(argv[1], "update")){
+    rv = runupdate(argc, argv);
   } else if(!strcmp(argv[1], "remove")){
     rv = runremove(argc, argv);
   } else if(!strcmp(argv[1], "convert")){
@@ -76,6 +80,7 @@ static void usage(void){
   fprintf(stderr, "  %s create [-fts] dbpath [scale]\n", g_progname);
   fprintf(stderr, "  %s import [-suf str] dbpath file ... \n", g_progname);
   fprintf(stderr, "  %s export [-dir str] dbpath [id]\n", g_progname);
+  fprintf(stderr, "  %s update id [file]\n", g_progname);
   fprintf(stderr, "  %s remove dbpath id\n", g_progname);
   fprintf(stderr, "  %s convert [-fw|-ft] [-buri str] [-duri] [-page] [file]\n", g_progname);
   fprintf(stderr, "  %s passwd [-salt str] [-info str] name pass\n", g_progname);
@@ -185,6 +190,44 @@ static int runexport(int argc, char **argv){
 }
 
 
+/* parse arguments of update command */
+static int runupdate(int argc, char **argv){
+  char *dbpath = NULL;
+  char *idstr = NULL;
+  char *file = NULL;
+  for(int i = 2; i < argc; i++){
+    if(!dbpath && argv[i][0] == '-'){
+      usage();
+    } else if(!dbpath){
+      dbpath = argv[i];
+    } else if(!idstr){
+      idstr = argv[i];
+    } else if(!file){
+      file = argv[i];
+    } else {
+      usage();
+    }
+  }
+  if(!dbpath || !idstr) usage();
+  int64_t id = tcatoi(idstr);
+  char *ibuf;
+  int isiz;
+  if(file && file[0] == '@'){
+    isiz = strlen(file) - 1;
+    ibuf = tcmemdup(file + 1, isiz);
+  } else {
+    ibuf = tcreadfile(file, -1, &isiz);
+  }
+  if(!ibuf){
+    eprintf("%s: cannot open", file ? file : "(stdin)");
+    return 1;
+  }
+  int rv = procupdate(dbpath, id, ibuf);
+  tcfree(ibuf);
+  return rv;
+}
+
+
 /* parse arguments of remove command */
 static int runremove(int argc, char **argv){
   char *dbpath = NULL;
@@ -491,6 +534,31 @@ static int procexport(const char *dbpath, int64_t id, const char *dirpath){
 }
 
 
+/* perform update command */
+static int procupdate(const char *dbpath, int64_t id, const char *wiki){
+  TCTDB *tdb = tctdbnew();
+  if(!tctdbopen(tdb, dbpath, TDBOWRITER)){
+    printdberr(tdb);
+    tctdbdel(tdb);
+    return 1;
+  }
+  bool err = false;
+  TCMAP *cols = tcmapnew2(TINYBNUM);
+  wikiload(cols, wiki);
+  if(!dbputart(tdb, id, cols)){
+    printdberr(tdb);
+    err = true;
+  }
+  tcmapdel(cols);
+  if(!tctdbclose(tdb)){
+    printdberr(tdb);
+    err = true;
+  }
+  tctdbdel(tdb);
+  return err ? 1 : 0;
+}
+
+
 /* perform remove command */
 static int procremove(const char *dbpath, int64_t id){
   TCTDB *tdb = tctdbnew();
